/**
 * Copyright (C) 2006-2017 INRIA and contributors
 * Spoon - http://spoon.gforge.inria.fr/
 *
 * This software is governed by the CeCILL-C License under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as
 * circulated by CEA, CNRS and INRIA at http://www.cecill.info.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the CeCILL-C License for more details.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package spoon.reflect.declaration;

import spoon.support.DerivedProperty;

/**
 * This interface represents a member of a class (field, method,
 * nested class or static/instance initializer).
 */
public interface CtTypeMember extends CtModifiable, CtNamedElement {

	/**
	 * Gets the type that declares this class member.
	 *
	 * @return declaring class
	 */
	@DerivedProperty
	CtType<?> getDeclaringType();

	/**
	 * Returns the top level type declaring this type if an inner type or type member.
	 * If this is already a top-level type, then returns itself.
	 */
	@DerivedProperty
	<T> CtType<T> getTopLevelType();
}
